import textwrap
from typing import List
import json
class StringUtil:
    @staticmethod
    def fecth_string(s) -> (str,str):
        if len(s) <= 256:
            return s,None
        else:
            head = s[:256]
            tail = s[256:]
            last_period = head.rfind('.')
            last_newline = head.rfind('\n')
            if last_period != -1 or last_newline != -1:
                end = max(last_period, last_newline)
                tail = head[end + 1:] + tail
                head = head[:end + 1]
            return head,tail
    @staticmethod
    def split_long_str(s) -> list[str]:
        chunks = []
        head, tail = StringUtil.fecth_string(s)
        if (head is not None and head.rstrip() != ''):
            chunks.append(head)
        while tail is not None:
            head, tail = StringUtil.fecth_string(tail)
            if (head is not None and head.rstrip() != ''):
                chunks.append(head)
        return chunks

    @staticmethod
    def is_valid_email(email):
        import re
        email_regex = r'\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b'
        return re.match(email_regex, email) is not None

class JsonEncoder(json.JSONEncoder):
    from datetime import datetime
    from bson import ObjectId
    def default(self, obj):
        if isinstance(obj, datetime):
            return str(obj)
        elif isinstance(obj, ObjectId):
            return str(obj)
        else:
            return super(JsonEncoder, self).default(obj)
if __name__ == '__main__':
    s="""
    The Zoho SalesIQ REST API uses the OAuth 2.0 protocol to authorize and authenticate calls. OAuth is an industry open standard for authorization. It provides secure access to protected resources, thereby reducing the hassle of asking for a username and a password every time you log in.
    In your request for access, you can request a refresh token to be returned along with the access token. A refresh token provides your app access to Rest APIs even when the user is not logged in. To request a refresh token, add access_type=offline to the authentication request.

The Prompt=consent will always generate the refresh token. The maximum number of refresh tokens is 20. Once the limit is reached, the first refresh token in terms of time will be deleted.

The Access Tokens have limited validity. In most general cases, the access tokens expire in an hour. Until then, the access token has unlimited usage. Once it expires, your app will have to use the refresh token to request a new access token.

For this new request, the parameters to be included are,
    """

    chuncks = StringUtil.split_long_str(s)
    for i, chunk in enumerate(chuncks):
        print(f"Chunk {i+1}:\n{chunk}\n")



from redis import StrictRedis
import sys

sys.path.append('../../')
from inhand.service.SettingService import Redis
from inhand.dto.ErrorException import ErrorCodeException

class CacheService:
    def __init__(self,settings):
        self.settings = settings

    def connect(self):
        pass

    def close(self):
        pass

    def put(self,key,value,expire=None):
        raise (NameError, "Unsupported!")

    def get(self, key):
        raise (NameError, "Unsupported!")

    def exists(self, key):
        raise (NameError, "Unsupported!")

    @staticmethod
    def createCacheService(settings):
        if isinstance(settings,Redis):
            return RedisCacheService(settings)
        else:
            raise (NameError, "Unsupported cache type: {}".format(database.type))

class RedisCacheService(CacheService):

    def __init__(self,settings):
        super().__init__(settings)
        self.cache=StrictRedis(host=settings.host,
                             port=settings.port,
                             db=settings.database,
                             password=settings.password)

    def put(self,key,value,expire=None):
        if expire is not None:
            self.cache.setex(key,expire,value)
        else:
            self.cache.set(key,value)

    def get(self, key):
        if self.exists(key):
            return  self.cache.get(key)
        else:
            raise ErrorCodeException(404,"no such key")

    def exists(self, key):
        return self.cache.exists(key)


#coding=utf-8
import http.client
import json
import sys
sys.path.append('../../')
from inhand.utilities.URLUtility import GenericURL
from inhand.utilities.Utility import Utility
import yaml

class SpringCloudSettings:
    def __init__(self):
        self.name=None
        self.profile=[]
        self.port=5000
        self.config_uri=None
        self.config_profile=None
        self.label=None
        self.version=None
        self.state=None
        self.params={}
        self.common={}
        self.consul=None
        self.listen_ip=None

    def __readBootstrapSettings(self,config):
        #config有可能是yaml也可能是json文件
        #with open(bootstrap_yaml,'r',encoding='utf-8' as f:
        #    config=yaml.load(f.read(),Loader=yaml.FullLoader))
        self.name=config['spring']['application']['name']
        self.config_uri=config['spring']['cloud']['config']['uri']
        self.config_profile=config['spring']['cloud']['config']['profile']
        self.port=int(config['server']['port'])
        self.consul=Consul(config)

    def readBootstrapYaml(self,bootstrap_yaml):
        #config有可能是yaml也可能是json文件
        with open(bootstrap_yaml,'r',encoding='utf-8') as f:
            config=yaml.load(f.read(),Loader=yaml.FullLoader)
        self.__readBootstrapSettings(config)

    def readBootstrapJson(self,bootstrap_json):
        #config有可能是yaml也可能是json文件
        with open(bootstrap_json,'r',encoding='utf-8') as f:
            config=json.load(f)
        self.__readBootstrapSettings(config)


    def fetch_settings(self,config_url="http://10.5.16.213:8801/tube-stat-api/dev",module="tube-stat-api",profile="dev"):

        url = GenericURL(config_url)
        resource = ("%s" % url.path)
        if url.scheme == 'http':
            conn = http.client.HTTPConnection(url.host)  # httplib.HTTPConnection(host=self.url.host,timeout=30)
        else:
            conn = http.client.HTTPSConnection(url.host)  # httplib.HTTPSConnection(host=self.url.host,timeout=30)

        data=dict()
        print("Access %s" % config_url)
        conn.request("GET",resource)
        ret = conn.getresponse()

        s = json.loads(ret.read())
        self.__readSettings(s,module,profile)

    #for testing on ci server
    def readLocalSettings(self,path="../../../test.json",module="tube-stat-api", profile="dev"):
        with open(path) as f:
            s = json.load(f, encoding='utf-8')
        self.__readSettings(s,module,profile)


    def __readSettings(self,s,module="tube-stat-api", profile="dev"):
        self.name = s['name']
        self.profile=s['profiles']
        self.label = s['label']
        self.version=s['version']
        self.state=s['state']

        isPrivate = False
        filename = module +"-"+profile
        print('****************** filename: {} ********************'.format(filename))
        for item in s['propertySources']:
            if filename in item['name']:
                isPrivate = True
            else:
                isPrivate = False
            for k in item['source']:
                if k not in self.params.keys():
                    self.params[k] = item['source'][k]
                else:
                    if isPrivate:
                        self.params[k] = item['source'][k]


    def fetchArraySize(self,key_regex):
        size = 0
        key=key_regex.format(size)
        while key in self.params:
            size=size+1
            key=key_regex.format(size)
        return size

class Mongo:
    def __init__(self,params):
        prefix = 'spring.data.mongodb.%s'
        self.host = params[(prefix % ('host'))]
        self.port = params[(prefix % ('port'))]
        self.auth_mechanism =params[(prefix % ('authMechanism'))]
        self.auth_db = params[(prefix % ('authentication-database'))]
        self.account = params[(prefix % ('username'))]
        self.password=params[(prefix % ('password'))]
        key = (prefix % ('maxPoolSize'))
        self.maxPoolSize = 20 if key not in params.keys() else int(params[key])
        key = (prefix % ('waitQueueMultiple'))
        self.waitQueueMultiple = 10 if key not in params.keys() else int(params[key])

class RDB_JDBC:
    def __init__(self,params):
        prefix = 'spring.database.%s'
        #'jdbc:mysql://mysql:3306/iwos-app-ai'
        url=params[(prefix % ('url'))]
        args=url.split(':')
        self.type=args[1]
        self.host = args[2][2:]
        strs=args[3].split('/')
        self.port = int(strs[0])
        strs1=strs[1].split('?')
        self.database=strs1[0]
        self.account = params[(prefix % ('username'))]
        self.password=params[(prefix % ('password'))]

class Zk:
    def __init__(self,params):
        prefix = "spring.cloud.zookeeper.%s"
        self.url = params[(prefix % ('connect-string'))]
        self.path = params[(prefix % ('discovery.root'))]

"""
"config.mq.host": "rabbit",
"config.mq.account": "admin",
"config.mq.password": "1qaz2wsx",
"config.mq.port": 5672,
"""

class MQ:
    def __init__(self,params):
        prefix = "spring.rabbitmq.%s"
        self.host = params[(prefix % ('host'))]
        self.port = int(params[(prefix % ('port'))])
        self.account = params[(prefix % ('username'))]
        self.password = params[(prefix % ('password'))]


# class RDB:
#     def __init__(self,params):
#         prefix = "config.app.settings.database.%s"
#         self.type = params[(prefix % ('type'))]
#         self.database = params[(prefix % ('database'))]
#         self.host = params[(prefix % ('host'))]
#         self.port = int(params[(prefix % ('port'))])
#         self.account = params[(prefix % ('account'))]
#         self.password = params[(prefix % ('pawssword'))]


"""
"spring.redis.host": "10.5.0.27",
"spring.redis.port": 6379,
"spring.redis.database": 0,
"spring.redis.password": "1qaz2wsx",
"spring.redis.pool.max-active": 8,
"spring.redis.pool.max-wait": -1,
"spring.redis.pool.max-idle": 9,
"spring.redis.pool.min-idle": 0,
"spring.redis.timeout": 0,
"""
class Redis:
    def __init__(self, params):
        prefix = 'spring.redis.%s'
        self.host=params[(prefix % ('host'))]
        self.port=int(params[(prefix % ('port'))])
        self.database=int(params[(prefix % ('database'))])
        self.password=params[(prefix % ('password'))]
        key=(prefix % ('timeout'))
        self.timeout=0 if key not in params.keys() else  int(params[key])

class Consul:
    def __init__(self,params):
        self.host=params['spring']['cloud']['consul']['host']
        self.port=int(params['spring']['cloud']['consul']['port'])
        self.application=params['spring']['cloud']['consul']['application']
        self.discovery=params['spring']['cloud']['consul']['discovery']['enable']
        self.instance_group=params['spring']['cloud']['consul']['discovery']['instance-group']
        self.health_check_path=params['spring']['cloud']['consul']['discovery']['health-check-path']
        self.health_check_interval=params['spring']['cloud']['consul']['discovery']['health-check-interval']

if __name__ == '__main__':
    settings=SpringCloudSettings()
    settings.readBootstrapYaml('d:/bootstrap.yml')
    print(settings.name)








#encoding=utf-8
import pika
import logging,traceback

class RabbitMQ:
    def __init__(self,mq_settings):
        self.settings=mq_settings
        self.conn=None

    def __connect(self):
        if self.conn is None:
            credentials = pika.PlainCredentials(self.settings.account, self.settings.password)
            self.conn=pika.BlockingConnection(pika.ConnectionParameters(self.settings.host,self.settings.port,'/',credentials))

    def send_msg(self,exchange,routing_key,message):
        try:
            self.__connect()
            channel = self.conn.channel()
            #channel.exchange_declare(exchange=exchange, durable=True, exchange_type='fanout')
            #channel.basic_publish(exchange=exchange, routing_key=routing_key, body=message,  mandatory=True,properties=pika.BasicProperties(delivery_mode = 2))

            channel.basic_publish(exchange=exchange, routing_key=routing_key, body=message)
            self.conn.close()
            self.conn = None
            return 200
        except Exception as e:
            traceback.print_exc()
            logging.warning(e,'while sending message to exchange: {} with key: {}'.format(exchange,routing_key))
            try:
                self.conn.close()
            finally:
                self.conn=None
                return 500





ERR_CODE_400 = 400
ERR_CODE_404 = 404
ERR_CODE_401 = 401
ERR_CODE_403 = 403
ERR_CODE_500 = 500
ERR_CODE_502 = 502

class Error:
    def __init__(self,errorCode, desc):
        self.error_code = errorCode
        self.error = desc
        self.status= errorCode

class APIError(Exception):
    def __init__(self, errCode, errorInfo):
        self.errorinfo = errorInfo
        self.errorCode = errCode
        self.status = errCode

    def __str__(self):
        return '{}(ErrCode={})'.format(self.errorinfo, str(self.errorCode))

class ErrorCodeException(APIError):
    def __init__(self, errCode, errorInfo):
        super().__init__(errCode,errorInfo)

    def __str__(self):
        return 'ErrCode: {}, {}'.format(self.errorinfo, str(self.errorCode))


import json
import rfc3339
from datetime import datetime
import time
import pytz
class InstanceInfo:
    def __init__(self,name="stat-api",startupAt=None,version_file=None):
        self.name=name
        self.startupAt=startupAt
        self.date = startupAt
        with open(version_file) as ss:
            s = json.load(ss)
        self.info=s

    def toMap(self):
        dt = datetime.now(pytz.UTC)
        return {"name": self.name,"startupAt": self.startupAt.strftime("%Y-%m-%dT%H:%M:%SZ"),"date":dt.strftime("%Y-%m-%dT%H:%M:%SZ"),"info":self.info}

#zookeeper中service的注册信息
"""
Ex. :
{
    "uriSpec": {
        "parts": [
            {
                "variable": true,
                "value": "scheme"
            },
            {
                "variable": false,
                "value": "://"
            },
            {
                "variable": true,
                "value": "address"
            },
            {
                "variable": false,
                "value": ":"
            },
            {
                "variable": true,
                "value": "port"
            }
        ]
    },
    "serviceType": "DYNAMIC",
    "registrationTimeUTC": 1546512992955,
    "port": 80,
    "payload": {
        "name": "iauth",
        "metadata": {
            "git.total.commit.count": "117",
            "git.tags": "v1.5.1",
            "git.remote.origin.url": "https://gitlab.inhand.local/elements/iauth",
            "git.commit.user.name": "j3r0lin",
            "git.commit.user.email": "j3r0lin@icloud.com",
            "git.commit.time": "2018-12-24T09:27:47Z",
            "git.commit.message.short": "DM-243 allow create user by phone number",
            "git.commit.message.full": "DM-243 allow create user by phone number",
            "git.commit.id.full": "7cec61d6797a897786ed087832dc58f70a3ef2fe",
            "git.commit.id.abbrev": "7cec61d",
            "git.closest.tag.name": "v1.5.1",
            "git.build.version": "v1.5.1",
            "git.build.time": "2018-12-24T10:53:31Z",
            "git.branch": "v1.5.1",
            "build.version": "v1.5.1",
            "build.time": "2018-12-24T10:53:31Z",
            "build.name": "iauth",
            "build.artifact": "iauth"
        },
        "id": "iauth:80",
        "@class": "org.springframework.cloud.zookeeper.discovery.ZookeeperInstance"
    },
    "name": "iauth",
    "id": "b89ddcfa-61cb-44da-8a21-29e13c06d801",
    "address": "172.18.0.3"
}
"""
class ServiceInfo:
    def __init__(self,id="",name="",address="0.0.0.0",port=80,serviceType="DYNAMIC",metadata={},class_desc="org.springframework.cloud.zookeeper.discovery.ZookeeperInstance"):
        self.uriSpec={"parts":[{
                "variable": True,
                "value": "scheme"
            },
            {
                "variable": False,
                "value": "://"
            },
            {
                "variable": True,
                "value": "address"
            },
            {
                "variable": False,
                "value": ":"
            },
            {
                "variable": True,
                "value": "port"
            }]}
        self.serviceType=serviceType
        self.registraionTimeUTC= int(time.mktime(datetime.now().timetuple()))
        self.port= port
        self.address=address
        self.id=id
        self.name=name
        self.metadata=metadata
        self.class_desc=class_desc

    def toMap(self):
        return {"id":self.id,"name":self.name,"address":self.address,"serviceType":self.serviceType,"registrationTimeUTC":self.registraionTimeUTC,
                "port":self.port,"uriSpec":self.uriSpec,"sslPort":None,"payload":{"name":self.name,"id":self.id,"@class":self.class_desc,"metadata":self.metadata}}


class ApisInfo:
    def __init__(self,url="",weight=100,id="",name="",apis=[]):
        self.url=url
        self.weight=weight
        self.name=name
        self.id=id
        self.apis=apis

    def toMap(self):
        return {"id":self.id,"name":self.name,"server":{"url":self.url,"weight":self.weight},"apis":self.apis}


import traceback
import psycopg2
import pymssql
import logging
from decimal import Decimal


class RDBService:
    def __init__(self, database):
        self.conn = None
        self.database = database

    def connect(self):
        self.__connect()

    def __connect(self):
        if self.conn is not None:
            return
        if self.database.type == 'mssql':
            self.conn = pymssql.connect(server=self.database.host,
                                        port=self.database.port,
                                        user=self.database.account,
                                        database=self.database.database,
                                        password=self.database.password)
        elif self.database.type == 'mysql':
            self.conn = pymssql.connect(server=self.database.host,
                                        port=self.database.port,
                                        user=self.database.account,
                                        database=self.database.database,
                                        password=self.database.password)
        elif self.database.type == 'postgresql':
            self.conn = psycopg2.connect(host=self.database.host,
                                         port=self.database.port,
                                         user=self.database.account,
                                         database=self.database.database,
                                         password=self.database.password)
        else:
            raise (NameError, "Unsupported!")

    def close(self):
        if self.conn != None:
            try:
                self.conn.close()
            finally:
                self.conn = None
                logging.info('Close the connnection with server.')

    # 向${collection}表新增一条记录${data}
    def insert(self, table, data):
        try:
            if self.conn is None:
                self.__connect()
            fields = []
            values = []
            for key in data:
                fields.append(key)
                values.append(self.__toValue(data[key]))
            sql = 'insert into {} ({}) values ({})'.format(table, ','.join(fields), ','.join(values))
            cusor = self.conn.cursor()
            cusor.execute(sql)
            self.conn.commit()
            cusor.close()
            # logging.debug("New a record into {}: {}".format(table, data))
        except Exception as e:
            traceback.print_exc()
            logging.warning('while excute insert new data({}) in {}'.format(table, data))
            self.close()
            raise e

    # 根据${query}查找一条记录
    """
    query = "where a=b or (c like 'd')" sql子句
    """

    def find_one(self, table, query, sort=None, fields=None):
        raise (NameError, "Unsupported")

    # 根据${query}查找多条记录
    """
    query = "where a=b or (c like 'd')" sql子句
    sort : 'order by ${fields} ${asc|desc}
    page_site: 分页大小 
    """

    def find(self, table, query, sort, fields, limit=None):
        raise (NameError, "Unsupported")

    # 根据${query}分页查找多条记录
    """
    query = "where a=b or (c like 'd')" sql子句
    sort : 'order by ${fields} ${asc|desc}
    page_size: 分页大小 
    page_number: 第几页
    """

    def findWithPages(self, table, query, sort, fields, page_number=1, page_size=10):
        raise (NameError, "Unsupported")

    """
    更新最先匹配filter的所有记录
    update: {${field}:${value}}
    """

    def update(self, table, query, update):
        try:
            if self.conn is None:
                self.__connect()
            expressions = []
            for key in update:
                expressions.append("{}={}".format(key, self.__toValue(update[key])))
            update_str = ",".join(expressions)
            sql = 'update {} set {} {}'.format(table, update_str, query)
            cusor = self.conn.cursor()
            cusor.execute(sql)
            self.conn.commit()
            cusor.close()

        except Exception as e:
            traceback.print_exc()
            logging.warning('while excute updating({}) sql with query({}) in {}'.format(update, table, query))
            self.close()
            raise e

    def delete(self, table, query):
        try:
            if self.conn is None:
                self.__connect()
            sql = 'delete from {} {}'.format(table, query)
            cusor = self.conn.cursor()
            res = cusor.execute(sql)
            self.conn.commit()
            cusor.close()
            # logging.debug('数据删除成功！')
        except Exception as e:
            traceback.print_exc()
            logging.warning('while excute deleting sql with query({}) in {}'.format(table, query))
            self.close()
            raise e

    def drop_table(self, table):
        try:
            if self.conn is None:
                self.__connect()
            sql = 'drop table if exists {}'.format(table)
            cusor = self.conn.cursor()
            res = cusor.execute(sql)
            self.conn.commit()
            cusor.close()
            # logging.debug('数据删除成功！')
        except Exception as e:
            traceback.print_exc()
            logging.warning('while excute droppping {}'.format(table))
            self.close()
            raise e

    def delete(self, table, query):
        try:
            if self.conn is None:
                self.__connect()
            sql = 'delete from {} {}'.format(table, query)
            cusor = self.conn.cursor()
            res = cusor.execute(sql)
            self.conn.commit()
            cusor.close()
            # logging.debug('数据删除成功！')
        except Exception as e:
            traceback.print_exc()
            logging.warning('while excute deleting sql with query({}) in {}'.format(table, query))
            self.close()
            raise e

    def count(self, table, query):
        try:
            if self.conn is None:
                self.__connect()
            countSql = 'select count(*) from {} {}'.format(table, query)
            cusor = self.conn.cursor()

            count = cusor.execute(countSql)
            count = cusor.fetchall()
            cusor.close()


        except Exception as e:
            traceback.print_exc()
            logging.warning('while excute counting sql with query({}) in {}'.format(table, query))
            self.close()
            raise e

    def __toValue(self, var):
        if isinstance(var, int) or isinstance(var, float):
            return str(var)
        elif isinstance(var, Decimal):
            # return str(round(var,2))
            return str(var)
        else:
            return '\'{}\''.format(var)

    @staticmethod
    def createRDBService(database):
        if database.type == 'mssql':
            return MssqlService(database)
        elif database.type == 'mysql':
            return MysqlService(database)
        elif database.type == 'postgresql':
            return PostgresqlService(database)
        else:
            raise (NameError, "Unsupported database type: {}".format(database.type))


class MssqlService(RDBService):
    def __init__(self, database):
        super().__init__(database)

    def __connect(self):
        super.connnect()

    def __close(self):
        super().close()

    # 根据${query}查找一条记录
    """
    query = "where a=b or (c like 'd')" sql子句
    """

    def find_one(self, table, query, sort=None, fields=None):
        try:
            self.__connect()
            s = '*'

            if fields is not None:
                s = ','.join(fields)

            sql = 'select top 1 {} from {} {} {}'.format(s, table, query if query is not None else '',
                                                         sort if sort is not None else '')
            cusor = self.conn.cursor()
            if not cusor:
                raise (NameError, "连接数据库失败")
            else:
                res = cusor.execute(sql)
                res = cusor.fetchall()
                cusor.close()
                return res[0] if len(res) > 0 else None
        except Exception as e:
            self.close()
            logging.warning('while excute find_one sql with query({}) in {}'.format(table, query))
            raise e

    # 根据${query}查找多条记录
    """
    query = "where a=b or (c like 'd')" sql子句
    sort : 'order by ${fields} ${asc|desc}
    page_size: 分页大小
    """

    def find(self, table, query, sort, fields, limit=None):
        try:
            self.__connect()
            topN = '' if limit is None else 'top {}'.format(limit)
            if fields is None:
                sql = 'select {} * from {} {} {}'.format(topN, table, query if query is not None else '',
                                                         sort if sort is not None else '')
            else:
                sql = 'select {} {} from {} {} {}'.format(topN, ','.join(fields), table,
                                                          query if query is not None else '',
                                                          sort if sort is not None else '')
            cusor = self.conn.cursor()
            if not cusor:
                raise (NameError, "连接数据库失败")
            else:
                cusor.execute(sql)
                result = cusor.fetchall()
                cusor.close()

                # return {'total': count, 'result': result}
                return result
        except Exception as e:
            self.close()
            logging.warning('while excute find sql with query({}) in {}'.format(table, query))
            raise e

    def findWithPages(self, table, query, sort, fields, page_number=1, page_size=10):
        try:
            self.__connect()

            page_params = '' if page_number is None or page_size is None \
                else 'offset {} rows fetch next {} rows only'.format((page_number-1) * page_size, page_size)
            if fields is None:
                sql = 'select * from {} {} {} {}'.format(table, query if query is not None else '',
                                                         sort if sort is not None else '', page_params)
            else:
                sql = 'select {} from {} {} {} {}'.format(','.join(fields), table, query if query is not None else '',
                                                          sort if sort is not None else '', page_params)
            cusor = self.conn.cursor()
            if not cusor:
                raise (NameError, "连接数据库失败")
            else:
                cusor.execute(sql)
                result = cusor.fetchall()
                cusor.close()

                # return {'total': count, 'result': result}
                return result
        except Exception as e:
            self.close()
            logging.warning('while excute find sql with query({}) in {}'.format(table, query))
            raise e


class MysqlService(RDBService):
    def __init__(self, settings):
        super().__init__(settings)

    def __connect(self):
        super().connect()

    def __close(self):
        super().close()

    # 根据${query}查找一条记录
    """
    query = "where a=b or (c like 'd')" sql子句
    """

    def find_one(self, table, query, sort=None, fields=None):
        try:
            self.__connect()
            s = '*'

            if fields is not None:
                s = ','.join(fields)

            sql = 'select {} from {} {} {} limit 1'.format(s, table, query if query is not None else '',
                                                           sort if sort is not None else '')
            cusor = self.conn.cursor()
            res = cusor.execute(sql)
            res = cusor.fetchall()
            cusor.close()
            return res[0] if len(res) > 0 else None
        except Exception as e:
            self.__close()
            logging.warning('while excute find_one sql with query({}) in {}'.format(table, query))
            raise e

    # 根据${query}查找多条记录
    """
    query = "where a=b or (c like 'd')" sql子句
    sort : 'order by ${fields} ${asc|desc}
    page_site: 分页大小 
    """

    def find(self, table, query, sort, fields, limit=None):
        try:
            self.__connect()

            topN = '' if limit is None else 'limit {}'.format(limit)
            if fields is None:
                sql = 'select  * from {} {} {} {}'.format(table, query if query is not None else '',
                                                          sort if sort is not None else '', topN)
            else:
                sql = 'select {} from {} {} {} {}'.format(','.join(fields), table, query if query is not None else '',
                                                          sort if sort is not None else '', topN)
            cusor = self.conn.cursor()
            result = cusor.execute(sql)
            result = cusor.fetchall()
            cusor.close()

            # return {'total': count,'result': result}
            return result
        except Exception as e:
            self.__close()
            logging.warning('while excute find sql with query({}) in {}'.format(table, query))
            raise e

    def findWithPages(self, table, query, sort, fields, page_number=1, page_size=10):
        try:
            self.__connect()
            page_params = '' if page_number is None or page_size is None \
                else 'limit {} offset {}'.format(page_size,(page_number-1) * page_size)
            if fields is None:
                sql = 'select * from {} {} {} {}'.format(table, query if query is not None else '',
                                                         sort if sort is not None else '', page_params)
            else:
                sql = 'select {} from {} {} {} {}'.format(','.join(fields), table, query if query is not None else '',
                                                          sort if sort is not None else '', page_params)
            cusor = self.conn.cursor()
            if not cusor:
                raise (NameError, "连接数据库失败")
            else:
                cusor.execute(sql)
                result = cusor.fetchall()
                cusor.close()

                # return {'total': count, 'result': result}
                return result
        except Exception as e:
            self.close()
            logging.warning('while excute find sql with query({}) in {}'.format(table, query))
            raise e

class PostgresqlService(MysqlService):
    def __init__(self, settings):
        super().__init__(settings)

    def __connect(self):
        super().connect()

    def __close(self):
        super().close()

    def drop_table(self, table):
        try:
            if self.conn is None:
                self.__connect()
            sql = 'drop table if exists "{}"'.format(table)
            cusor = self.conn.cursor()
            res = cusor.execute(sql)
            self.conn.commit()
            cusor.close()
            # logging.debug('数据删除成功！')
        except Exception as e:
            traceback.print_exc()
            logging.warning('while excute droppping {}'.format(table))
            self.close()
            raise e





class RestAPIResult:
    def __init__(self,error:str=None,error_code:int=None,result:any=None):
        self.error=error
        self.error_code=error_code
        self.result=result
        self.status=error_code if error_code is not None else 200

class BasicResult(RestAPIResult):
    def __init__(self,error:str=None,error_code:int=None,result:any=None):
        super().__init__(error,error_code,result)


class PageResult(RestAPIResult):
    def __init__(self,error:str=None,error_code:int=None,total:int=0,pageNumber:int=1,
                 pageSize:int=None,result:list[dict]=None,stats:dict=None):
        super.__init__(error,error_code,result)

        self.total=total
        self.pageNumber=pageNumber
        self.pageSize=pageSize
        self.stats=stats

class ResultUtil:
    @staticmethod
    def parseBasicResult(result:dict) -> BasicResult:
        return BasicResult(result['error'],result['error_code'],result['result'])

    @staticmethod
    def parsePageResult(result:dict) -> PageResult:
        return PageResult(result['error'],result['error_code'],
                          result['total'],
                          result['pageNumber'] if 'pageNumber' in result.keys() else 1,
                          result['pageSize'] if 'pageSize' in result.keys() else None,
                          result['result'],
                          result['stats'] if 'stats' in result.keys() else None)

    @staticmethod
    def validate(field_value: any, error_msg: str, field_type: str = None):
        from inhand.dto.APIError import APIError
        from inhand.dto.ErrorException import ERR_CODE_400
        from inhand.utils.StringUtil import StringUtil
        if field_value is None:
            raise APIError(ERR_CODE_400, error_msg)
        if field_type == 'str':
            if str(field_value) == '':
                raise APIError(ERR_CODE_400, error_msg)
        elif field_type == 'email':
            if not StringUtil.is_valid_email(field_value):
                raise APIError(ERR_CODE_400, error_msg)




import json
from inhand.service.MQSender import RabbitMQ

class PushMessage:
    locale: str = "English"
    title: str = None
    short_content: str = None
    content: str = None
    app_tag_list: list = []
    app_alias_list: list = []
    audience: str = None
    phone_list: list = []
    email_list: list = []
    im_list: list = []
    params: dict = {}

    def __init__(self):
        pass

    def toDict(self):
        kv= {
            'locale': self.locale,
            'title': self.title,
            'short_content': self.short_content,
            'content': self.content,
            'app_tag_list': self.app_tag_list,
            'app_alias_list': self.app_alias_list,
            'audience': self.audience,
            'phone_list': self.phone_list,
            'email_list': self.email_list,
            'im_list': self.im_list,
            'params': self.params
        }
        return kv

    def toJson(self):
        return json.dumps(self.toDict())

    def send(self,mq_sender: RabbitMQ, exchange:str, routing_key:str):
        mq_sender.send_msg(exchange,routing_key,self.toJson())


if __name__ == '__main__':
    message = PushMessage()
    message.title = "Test"
    print(message.toJson())


import setuptools
from setuptools import setup
with open("README.md", "r") as fh:
    long_description = fh.read()

setup (
    name = 'ih_common',
    version = '2.16.15',
    author = 'hanchuanjun',
    author_email = 'han@inhand.com.cn',
    url = 'https://gitlab.inhand.design/hancj-dev/py-common',
    description = 'tool',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    packages=setuptools.find_packages(),
    install_requires=[
		#'pandas','numpy','iso8601','rfc3339','pytz','tzwhere','requests','lxml',minio==7.1.13 boto3==1.26.93
	],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)